from typing import List

from omnisdk.omnitron.endpoints import (ChannelCreateOrderEndpoint,
                                        ChannelIntegrationActionEndpoint,
                                        ChannelOrderShippingInfoEndpoint,
                                        ChannelOrderItemEndpoint,
                                        ChannelBatchRequestEndpoint,
                                        ChannelOrderEndpoint,
                                        ChannelCargoEndpoint,
                                        ChannelCancellationRequestEndpoint)
from omnisdk.omnitron.models import Order, OrderShippingInfo, \
    CancellationRequest

from channel_app.core.commands import OmnitronCommandInterface
from channel_app.core.data import (OrderBatchRequestResponseDto,
                                   OmnitronCreateOrderDto, OmnitronOrderDto,
                                   OrderItemDto, CancelOrderDto, ErrorReportDto)
from channel_app.core.utilities import split_list
from channel_app.omnitron.batch_request import ClientBatchRequest
from channel_app.omnitron.commands.batch_requests import ProcessBatchRequests
from channel_app.omnitron.constants import (ContentType, BatchRequestStatus)
from channel_app.omnitron.exceptions import OrderException


class GetOrders(OmnitronCommandInterface):
    endpoint = ChannelOrderEndpoint
    batch_service = ClientBatchRequest
    content_type = ContentType.order.value
    path = "updates"
    BATCH_SIZE = 100

    def get_data(self) -> List[Order]:
        orders = self.get_orders()
        return orders

    def get_orders(self) -> List[Order]:
        orders = self.endpoint(
            path=self.path, channel_id=self.integration.channel_id
        ).list(
            params={
                "limit": self.BATCH_SIZE
            }
        )
        orders = orders[:self.BATCH_SIZE]

        objects_data = self.create_batch_objects(
            data=orders, content_type=self.content_type)
        self.update_batch_request(objects_data=objects_data)

        return orders


class GetOrderItems(OmnitronCommandInterface):
    endpoint = ChannelOrderItemEndpoint

    def get_data(self):
        order = self.objects
        order_items = self.get_order_items(order)
        for order_item in order_items:
            order_item.content_type = ContentType.order_item.value
        self.integration.do_action(key="get_integration_with_object_id",
                                   objects=order_items)
        return order_items


class ProcessOrderBatchRequests(OmnitronCommandInterface, ProcessBatchRequests):
    """
    Not finalized batch requests with content type `order` is managed in this command
    """
    endpoint = ChannelBatchRequestEndpoint
    content_type = ContentType.order.value
    BATCH_SIZE = 100
    CHUNK_SIZE = 50

    def validated_data(self, data: List[OrderBatchRequestResponseDto]):
        for item in data:
            assert isinstance(item, OrderBatchRequestResponseDto)
        return data

    def send(self, validated_data):
        result = self.process_item(validated_data)
        return result

    @property
    def update_state(self, *args, **kwargs) -> BatchRequestStatus:
        return BatchRequestStatus.done

    def get_remote_order_number(self, obj, integration_actions):
        for integration_action in integration_actions:
            content_type = integration_action.content_type["model"]
            if content_type != "order":
                continue
            if integration_action.object_id == obj.pk:
                return integration_action.remote_id

    def get_channel_items_by_reference_object_ids(self, channel_response,
                                                  model_items_by_content,
                                                  integration_actions):
        channel_items_by_order_id = {}
        for order_id, order in model_items_by_content["order"].items():
            number = self.get_remote_order_number(
                obj=order, integration_actions=integration_actions)
            for channel_item in channel_response:
                # TODO: comment
                if channel_item.number != number:
                    continue
                remote_item = channel_item
                channel_items_by_order_id[order_id] = remote_item
                break
        return channel_items_by_order_id

    def get_orders(self, id_list) -> dict:
        if not id_list:
            return {}

        end_point = ChannelOrderEndpoint(
            channel_id=self.integration.channel_id)
        orders = []
        for chunk_id_list in split_list(id_list, self.CHUNK_SIZE):
            orders_batch = end_point.list(
                params={"id__in": ",".join(chunk_id_list),
                        "limit": len(chunk_id_list)})
            orders.extend(orders_batch)
        return {order.pk: order for order in orders}

    def group_model_items_by_content_type(self, items_by_content):
        batch_items = {}
        for model, model_pks in items_by_content.items():
            if model != "order":
                continue
            group_items = self.get_orders(model_pks)
            batch_items[model] = group_items
        return batch_items


class CreateOrders(OmnitronCommandInterface):
    endpoint = ChannelCreateOrderEndpoint
    content_type = ContentType.order.value
    CHUNK_SIZE = 50

    def get_data(self) -> dict:
        assert isinstance(self.objects, OmnitronCreateOrderDto)

        order = self.objects.order
        order_item = self.objects.order_item
        order_items = self.prepare_order_items(order_items=order_item)
        extra_field = self.get_extra_field(order)
        data = {
            "order_item": order_items,
            "order": {
                "number": order.number[:128],
                "channel": self.integration.channel_id,
                "customer": order.customer,
                "shipping_address": order.shipping_address,
                "billing_address": order.billing_address,
                "currency": order.currency,
                "amount": order.amount,
                "shipping_amount": order.shipping_amount,
                "shipping_tax_rate": order.shipping_tax_rate,
                "extra_field": extra_field,
                "delivery_type": order.delivery_type,
                "cargo_company": order.cargo_company,
                "discount_amount": order.discount_amount or 0,
                "net_shipping_amount": order.net_shipping_amount,
                "tracking_number": (order.tracking_number and
                                    order.tracking_number[:256]),
                "carrier_shipping_code": (order.carrier_shipping_code and
                                          order.carrier_shipping_code[:256]),
                "remote_addr": order.remote_addr,
                "has_gift_box": order.has_gift_box,
                "gift_box_note": order.gift_box_note[:160],
                "client_type": "default",
                "language_code": order.language_code,
                "notes": order.notes[:320],
                "delivery_range": order.delivery_range,
                "shipping_option_slug": order.shipping_option_slug[:128],
                "date_placed": str(order.created_at)
            }
        }
        return data

    def get_extra_field(self, order: OmnitronOrderDto):
        extra_field = order.extra_field or {}
        if "id" not in extra_field and order.remote_id:
            extra_field.update({"id": order.remote_id})
        return extra_field

    def send(self, validated_data) -> object:
        order_obj = Order(**validated_data)
        order_endpoint = ChannelOrderEndpoint
        try:
            order_number = order_obj.order.get("number")
            is_order_exists = order_endpoint(
                channel_id=self.integration.channel_id
            ).list(
                params={
                    "number": order_number,
                    "channel_id": self.integration.channel_id
                    }
            )
            if is_order_exists:
                raise OrderException(params="Order Already Exist On Omnitron")

        except OrderException:
            return is_order_exists

        order = self.endpoint(
            channel_id=self.integration.channel_id
        ).create(item=order_obj)

        self._update_batch_request(order)
        return order

    def normalize_response(self, data, response) -> List[object]:
        return [data]

    def _update_batch_request(self, order):
        order.remote_id = order.extra_field.get("id")
        objects_data_order = self.create_batch_objects(data=[order],
                                                       content_type=ContentType.order.value)
        order_items = self.get_order_items(order_pk=order.pk)
        for item in order_items:
            item.remote_id = item.extra_field["id"]
        objects_data_order_items = self.create_batch_objects(
            data=order_items,
            content_type=ContentType.order_item.value)
        objects_data = []
        objects_data.extend(objects_data_order)
        objects_data.extend(objects_data_order_items)
        self.update_batch_request(objects_data=objects_data)

    def get_order_items(self, order_pk):
        params = {"order": order_pk}
        endpoint = ChannelOrderItemEndpoint(
            channel_id=self.integration.channel_id)
        order_items = endpoint.list(params=params)
        for item in endpoint.iterator:
            if not item:
                break
            order_items.extend(item)
        return order_items

    @property
    def update_state(self, *args, **kwargs) -> BatchRequestStatus:
        return BatchRequestStatus.commit

    def prepare_order_items(self, order_items: List[OrderItemDto]):
        product_dict = self.get_products(order_items)
        items = []
        for item in order_items:
            price_list = self.integration.catalog.price_list
            stock_list = self.integration.catalog.stock_list
            order_item_data = {
                "product": product_dict[item.product],
                "status": item.status or "400",
                "price_currency": item.price_currency,
                "price": item.price,
                "tax_rate": item.tax_rate,
                "extra_field": {
                    "id": item.remote_id
                },
                "price_list": item.price_list or price_list,
                "stock_list": item.stock_list or stock_list,
                "tracking_number": item.tracking_number,
                "carrier_shipping_code": item.carrier_shipping_code[:256],
                "discount_amount": item.discount_amount,
                "retail_price": item.retail_price,
                "attributes": item.attributes or {},
                "attributes_kwargs": item.attributes_kwargs or {},
                "parent": item.parent,
                "delivered_date": item.delivered_date,
                "estimated_delivery_date": item.delivered_date,
            }
            items.append(order_item_data)
        return items

    def get_order_item_extra_field(self, order_item: OrderItemDto):
        extra_field = order_item.extra_field
        if "id" not in extra_field and order_item.remote_id:
            extra_field.update({"id": order_item.remote_id})
        return extra_field

    def get_products(self, order_items: List[OrderItemDto]) -> dict:
        product_remote_ids = self.get_product_remote_id_list(order_items)
        endpoint = ChannelIntegrationActionEndpoint(
            channel_id=self.integration.channel_id)
        product_integration_actions = []
        for chunk in split_list(product_remote_ids, self.CHUNK_SIZE):
            params = {"channel": self.integration.channel_id,
                      "content_type_model": ContentType.product.value,
                      "remote_id__in": ",".join(chunk)}
            ia = endpoint.list(params=params)
            product_integration_actions.extend(ia)
        return {ia.remote_id: ia.object_id for ia in
                product_integration_actions}

    def get_product_remote_id_list(self, order_items: List[OrderItemDto]):
        product_remote_ids = [item.product for item in order_items]
        return product_remote_ids


class CreateOrderShippingInfo(OmnitronCommandInterface):
    endpoint = ChannelOrderShippingInfoEndpoint

    def get_data(self) -> dict:
        order: Order = self.objects
        cargo_is_send = self.integration.channel.conf.get(
            "send_shipping_info", False)
        shipping_company = self.get_shipping_company(
            cargo_company_id=order.cargo_company)
        data = {
            "order": order.pk,
            "order_number": order.number,
            "shipping_company": shipping_company,
            "is_send": cargo_is_send
        }
        return data

    def get_shipping_company(self, cargo_company_id):
        endpoint = ChannelCargoEndpoint(channel_id=self.integration.channel_id)
        cargo_company = endpoint.retrieve(id=cargo_company_id)
        return cargo_company

    def send(self, validated_data) -> object:
        """
        :param validated_data: data for order
        :return: order objects
        """
        order_shipping_info_obj = OrderShippingInfo(**validated_data)
        order = self.endpoint(channel_id=self.integration.channel_id).create(
            item=order_shipping_info_obj)
        return order

    def check_run(self, is_ok, formatted_data):
        if not is_ok:
            return False
        return True


class CreateOrderCancel(OmnitronCommandInterface):
    endpoint = ChannelOrderEndpoint
    path = "{pk}/cancel"

    def get_data(self):
        assert isinstance(self.objects, CancelOrderDto)
        cancel_data = self.objects
        order_pk = self.get_order_pk(order_remote_id=cancel_data.order)
        order_item_dict = self.get_order_item_dict(
            cancel_items=cancel_data.cancel_items)
        order_item_pk_list = []
        for key, pk_list in order_item_dict.items():
            order_item_pk_list.extend(pk_list)

        reasons = self.get_reasons_data(order_item_dict=order_item_dict,
                                        reasons=cancel_data.reasons)

        data = {
            "cancel_items": order_item_pk_list,
            "order": order_pk,
            "reasons": reasons,
            "forced_refund_amount": cancel_data.forced_refund_amount
        }
        return data

    def get_order_pk(self, order_remote_id):
        end_point = ChannelIntegrationActionEndpoint(
            channel_id=self.integration.channel_id)
        params = {"channel": self.integration.channel_id,
                  "content_type_model": ContentType.order.value,
                  "remote_id": order_remote_id}
        integration_actions = end_point.list(params=params)
        if not integration_actions:
            raise Exception(
                "Order not found: number={}".format(order_remote_id))
        if len(integration_actions) != 1:
            raise Exception(
                "CancelOrder.get_order_pk query incorrect: params={}".format(
                    params))

        integration_action = integration_actions[0]
        object_id = integration_action.object_id
        return object_id

    def get_order_item_dict(self, cancel_items: List[str]) -> dict:
        """
        cancel_items: ["121", "232"] -> omnitron orderitem remote_id list
        :return: dict
            {orderitem_remote_id: [omnitron_id1, omnitronid2]}
            {"121": [1001]}
            {"232": [1002, 1003]}
        """
        end_point = ChannelIntegrationActionEndpoint(
            channel_id=self.integration.channel_id)

        object_ids = {}
        for order_item_remote_id in cancel_items:
            params = {"channel": self.integration.channel_id,
                      "content_type_model": ContentType.order_item.value,
                      "remote_id": order_item_remote_id}
            integration_actions = end_point.list(params=params)
            for item in end_point.iterator:
                if not item:
                    break
                integration_actions.extend(item)

            if not integration_actions:
                raise Exception(
                    "CreateOrderCancel: OrderItem not found, number={}".format(
                        order_item_remote_id))
            ids = [integration_action.object_id
                   for integration_action in integration_actions]

            object_ids[order_item_remote_id] = ids

        return object_ids

    def get_reasons_data(self, order_item_dict: dict, reasons: dict):
        """
        order_item_dict: dict  # {remote_basketitem_id: [id1,id2]}
        reasons: dict # {order_item_remote_id: remote_reason_code}
        """
        reasons_dict = {}  # order_item_id: reason_id
        for order_item_remote_id, reason_remote_code in reasons.items():
            order_item_id_list = order_item_dict[order_item_remote_id]
            reason_mapping = self.integration.channel.conf.get(
                "reason_mapping", {})
            try:
                cancellation_reason_id = reason_mapping[reason_remote_code]
            except KeyError:
                cancellation_reason_id = self.integration.channel.conf.get(
                    "cancellation_reason_id")

            for order_item_id in order_item_id_list:
                reasons_dict[order_item_id] = cancellation_reason_id
        return reasons_dict

    def send(self, validated_data) -> Order:
        path = self.path.format(pk=validated_data["order"])
        endpoint = self.endpoint(path=path,
                                 channel_id=self.integration.channel_id,
                                 raw=True)
        response = endpoint.create(item=validated_data)
        return response

    def normalize_response(self, data, response) -> List[object]:
        return [data]


class GetCancellationRequest(OmnitronCommandInterface):
    endpoint = ChannelCancellationRequestEndpoint
    content_type = ContentType.cancellation_request.value
    CHUNK_SIZE = 50

    def get_data(self):
        """
        {"status": "approved", "cancellation_type": "refund or cancel", "order_item": 3212}
        """
        assert isinstance(self.objects, dict)
        query_params = self.objects
        if not query_params.get("limit", None):
            query_params["limit"] = self.CHUNK_SIZE
        return self.get_cancellation_requests(query_params=query_params)

    def get_cancellation_requests(self, query_params={}) -> List[
        CancellationRequest]:
        endpoint = self.endpoint(path=self.path,
                                 channel_id=self.integration.channel_id)
        cancellation_requests = endpoint.list(query_params)
        for batch in endpoint.iterator:
            if not batch:
                break
            cancellation_requests.extend(batch)
        return cancellation_requests



from redis import Redis


class RedisClient(Redis):
    def __init__(self):
        from channel_app.core import settings
        self.broker_database_index = int(settings.CACHE_DATABASE_INDEX)
        self.broker_port = settings.CACHE_PORT
        self.broker_host = settings.CACHE_HOST
        super(RedisClient, self).__init__(host=self.broker_host,
                                          port=self.broker_port,
                                          db=self.broker_database_index)



import asyncio
from typing import Any

from omnisdk.omnitron.endpoints import CatalogEndpoint, ChannelEndpoint
from omnisdk.omnitron.models import Catalog, Channel


class BaseIntegration(object):
    """
    To integrate with any system you must create a class which inherits from BaseIntegration.
    This class was designed to work with `command design pattern` which basically defines
    a task procedure interface. All defined commands override some of the default base
    methods according to their requirements.
    """
    actions = {}

    def get_action(self, key: str):
        return self.actions[key]

    def do_action(self, key: str, **kwargs) -> Any:
        """
        Runs the command given with the key and supplies the additional parameters to the command.

        :param key: Command key
        :param kwargs: Any additional parameters can be specified, for example `objects` must be
            supplied if you want to provide input to the action.

        :return: Result of the command

        """
        action_class = self.get_action(key)
        action_object = action_class(integration=self, **kwargs)
        return action_object.run()

    def do_action_async_run(self, key: str, **kwargs) -> Any:
        """
        Runs the command given with the key asynchronously and supplies the additional parameters
        to the command.

        :param key: Command key
        :param kwargs: Any additional parameters can be specified, for example `objects` must be
            supplied if you want to provide input to the action.

        :return: Result of the command

        """

        action_class = self.get_action(key)
        action_object = action_class(integration=self, **kwargs)
        return asyncio.run(action_object.run_async())

    @property
    def catalog(self) -> Catalog:
        """
        Retrieves the catalog object using the `catalog_id` stored in the `self`.

        Side effect: It stores the result in the `self.catalog_object`, if catalog is updated
        on the currently running task you must delete self.catalog_object and re-call this method
        """
        if not getattr(self, 'catalog_object', None):
            self.catalog_object = CatalogEndpoint().retrieve(id=self.catalog_id)
        return self.catalog_object

    @property
    def channel(self) -> Channel:
        """
        Retrieves the channel object using the `channel_id` stored in the `self`.

        Side effect: It stores the result in the `self.channel_object`, if channel is updated
        on the currently running task you must delete self.channel_object and re-call this method
        """
        if not getattr(self, 'channel_object', None):
            self.channel_object = ChannelEndpoint().retrieve(id=self.channel_id)
        return self.channel_object



import functools
from collections import defaultdict


class ProductCommonMixin(object):
    def group_by_product_id(self, products):
        """
       # products were grouped by color which was grouped by color
       :param products: [product1,product2]
       :return: Dict {"base_code-color":[product1,product2]}
       """
        group = defaultdict(list)
        for p in products:
            key = self.get_product_id(p)
            group[key].append(p)
        return group

    def get_product_id(self, obj):
        """
        # Your internal product id to reference for
        # this product.
        :param obj:
        :return: String (required)
        """
        product_id = getattr(obj, "mapped_attributes", {}).get(
            "mapped_attributes", {}).get("product_id", None)
        if product_id:
            return product_id
        return self.get_color(obj)

    @staticmethod
    def get_reduce_data(code, value):
        try:
            data = functools.reduce(
                lambda d, key: d.get(key, None) if isinstance(d,
                                                              dict) else None,
                code.split("__"), value)
            return data
        except TypeError:
            return None

    def get_color(self, obj):
        """
        # Valid color provided by namshi
        :param obj:
        :return:String (required,namshi_lookup)
        """
        color = getattr(obj, "mapped_attributes", {}).get(
            "mapped_attributes", {}).get("color", obj.listing_code)
        color = color and obj.attributes["renk"]
        return color

    def get_barcode(self, obj):
        """
        # The barcode uniquely identifying a
        # certain product line across
        remote_id_attribute = ["attributes","barcode"], ["sku"]
        :param obj:
        :return: String (required)
        """
        remote_id_attribute = self.integration.channel.conf.get("remote_id_attribute")
        if remote_id_attribute:
            return self.get_reduce_data(remote_id_attribute, obj.__dict__)
        return obj.sku



from dataclasses import asdict
from omnisdk.omnitron.endpoints import ChannelCustomerEndpoint
from omnisdk.omnitron.models import Customer

from channel_app.core.commands import OmnitronCommandInterface
from channel_app.core.data import CustomerDto
from channel_app.omnitron.constants import CustomerIdentifierField


class GetOrCreateCustomer(OmnitronCommandInterface):
    endpoint = ChannelCustomerEndpoint

    def get_data(self) -> Customer:
        data = self.objects
        data: CustomerDto
        return self.get_customer(data)

    def get_customer(self, data: CustomerDto) -> Customer:
        customer_identifier_field = self.integration.channel.conf.get(
            "CUSTOMER_IDENTIFIER_FIELD", CustomerIdentifierField.email)
        customers = None
        if customer_identifier_field == CustomerIdentifierField.email:
            customers = self.endpoint(channel_id=self.integration.channel_id).list(params={
                "email": data.email,
            })
            for c in customers:
                if c.email != data.email:
                    raise Exception("Customer email filter incorrect")
        elif customer_identifier_field == CustomerIdentifierField.phone_number:
            # TODO phone_number filter does not exist on customerfilter
            customers = self.endpoint(channel_id=self.integration.channel_id).list(params={
                "phone_number": data.phone_number,
            })
            for c in customers:
                if c.phone_number != data.phone_number:
                    raise Exception("Customer phone_number filter incorrect")

        if customers:
            customer = customers[0]
            must_update = False
            new_customer = Customer()
            new_customer.channel = customer.channel
            new_customer.channel_code = customer.channel_code
            if "email" in asdict(data) and data.email != customer.email:
                must_update = True
                new_customer.email = data.email
            if "phone_number" in asdict(data) and data.phone_number != customer.phone_number:
                must_update = True
                new_customer.phone_number = data.phone_number
            if "first_name" in asdict(data) and data.first_name != customer.first_name:
                must_update = True
                new_customer.first_name = data.first_name
            if "last_name" in asdict(data) and data.last_name != customer.last_name:
                must_update = True
                new_customer.first_name = data.last_name
            if must_update:
                customer = self.endpoint(channel_id=self.integration.channel_id).update(
                    id=customer.pk, item=customer)
        else:
            new_customer = Customer()
            new_customer.channel = self.integration.channel_id
            new_customer.email = data.email
            new_customer.phone_number = data.phone_number
            new_customer.first_name = data.first_name
            new_customer.last_name = data.last_name
            new_customer.extra_field = data.extra_field or {}
            new_customer.channel_code = data.channel_code
            new_customer.is_active = data.is_active or True
            new_customer = self.endpoint(channel_id=self.integration.channel_id).create(
                item=new_customer)
            customer = new_customer
        return [customer]



class IntegrationMappingException(Exception):
    def __init__(self, params=None):
        super(IntegrationMappingException, self).__init__()
        self.params = params


class CountryException(IntegrationMappingException):
    pass


class CityException(IntegrationMappingException):
    pass


class TownshipException(IntegrationMappingException):
    pass


class DistrictException(IntegrationMappingException):
    pass


class AddressException(IntegrationMappingException):
    pass


class OrderException(Exception):
    def __init__(self, params=None):
        super(OrderException, self).__init__()
        self.params = params


class CargoCompanyException(Exception):
    def __init__(self, params=None):
        super().__init__()
        self.params = params


class CustomerException(Exception):
    def __init__(self, params=None):
        super(CustomerException, self).__init__()
        self.params = params



import importlib
import os

env_variables = os.environ

OMNITRON_CHANNEL_ID = os.getenv("OMNITRON_CHANNEL_ID")
OMNITRON_USER = os.getenv("OMNITRON_USERNAME")
OMNITRON_PASSWORD = os.getenv("OMNITRON_PASSWORD")
MAIN_APP_URL = os.getenv("MAIN_APP_URL")
OMNITRON_URL = f"https://{MAIN_APP_URL}/"
OMNITRON_CATALOG_ID = os.getenv("OMNITRON_CATALOG_ID")
CACHE_DATABASE_INDEX = os.getenv("CACHE_DATABASE_INDEX")
CACHE_HOST = os.getenv("CACHE_HOST")
CACHE_PORT = os.getenv("CACHE_PORT")
BROKER_HOST = os.getenv("BROKER_HOST")
BROKER_PORT = os.getenv("BROKER_PORT")
BROKER_DATABASE_INDEX = os.getenv("BROKER_DATABASE_INDEX")
SENTRY_DSN = os.getenv("SENTRY_DSN")
DEFAULT_CONNECTION_POOL_COUNT = os.getenv("DEFAULT_CONNECTION_POOL_COUNT") or 10
DEFAULT_CONNECTION_POOL_MAX_SIZE = os.getenv("DEFAULT_CONNECTION_POOL_COUNT") or 10
DEFAULT_CONNECTION_POOL_RETRY = os.getenv("DEFAULT_CONNECTION_POOL_RETRY") or 0
REQUEST_LOG = os.getenv("REQUEST_LOG") or False

omnitron_module = importlib.import_module(os.environ.get("OMNITRON_MODULE"))
OmnitronIntegration = omnitron_module.OmnitronIntegration

channel_module = importlib.import_module(os.environ.get("CHANNEL_MODULE"))
ChannelIntegration = channel_module.ChannelIntegration



from typing import List

from omnisdk.omnitron.endpoints import (
    ChannelProductImageEndpoint, ChannelIntegrationActionEndpoint, ChannelBatchRequestEndpoint
)
from omnisdk.omnitron.models import ProductImage

from channel_app.core.commands import OmnitronCommandInterface
from channel_app.core.data import BatchRequestResponseDto
from channel_app.omnitron.commands.batch_requests import ProcessBatchRequests
from channel_app.omnitron.constants import ContentType, IntegrationActionStatus, FailedReasonType, BatchRequestStatus


class GetUpdatedProductImages(OmnitronCommandInterface):
    path = "updates"
    endpoint = ChannelProductImageEndpoint
    BATCH_SIZE = 100
    content_type = ContentType.product_image.value

    def get_data(self) -> List[ProductImage]:
        images = self.get_product_images()
        images = self.get_integration_actions(images)
        return images

    def get_product_images(self) -> List[ProductImage]:
        images = self.endpoint(
            path=self.path,
            channel_id=self.integration.channel_id
        ).list(
            params={
                "limit": self.BATCH_SIZE
            }
        )
        images = images[:self.BATCH_SIZE]
        objects_data = self.create_batch_objects(data=images,
                                                 content_type=self.content_type)
        self.update_batch_request(objects_data=objects_data)
        return images

    def get_integration_actions(self, images: List[ProductImage]):
        if not images:
            return []

        endpoint = ChannelIntegrationActionEndpoint(
            channel_id=self.integration.channel_id
        )
        image_integration_actions = endpoint.list(
            params={
                "local_batch_id": self.integration.batch_request.local_batch_id,
                "status": IntegrationActionStatus.processing,
                "channel_id": self.integration.channel_id
            }
        )
        for image_batch in endpoint.iterator:
            image_integration_actions.extend(image_batch)

        image_ia_dict = {ia.object_id: ia for ia in image_integration_actions}

        for image in images:
            image_ia = image_ia_dict[image.pk]
            image.remote_id = image_ia.remote_id

        return images


class GetInsertedProductImages(GetUpdatedProductImages):
    path = "inserts"

    def get_integration_actions(self, images: List[ProductImage]):
        if not images:
            return []
        endpoint = ChannelIntegrationActionEndpoint(
            channel_id=self.integration.channel_id)
        product_ids = [str(image.product) for image in images]
        product_ias = endpoint.list(
            params={"object_id__in": ",".join(product_ids),
                    "content_type_name": ContentType.product.value,
                    "status": IntegrationActionStatus.success,
                    "channel_id": self.integration.channel_id
                    })
        for product_batch in endpoint.iterator:
            product_ias.extend(product_batch)
        product_integrations_by_id = {ia.object_id: ia for ia in product_ias}

        for image in images:
            if image.product in product_integrations_by_id:
                product_ia = product_integrations_by_id[image.product]
                image.remote_id = product_ia.remote_id
            else:
                image.failed_reason_type = FailedReasonType.channel_app.value
                self.failed_object_list.append(
                    (image, ContentType.product_image.value,
                     "Product has not been sent"))
        return images


class ProcessImageBatchRequests(OmnitronCommandInterface, ProcessBatchRequests):
    endpoint = ChannelBatchRequestEndpoint
    content_type = ContentType.product_image.value
    CHUNK_SIZE = 50
    BATCH_SIZE = 100

    def get_data(self):
        return self.objects

    def validated_data(self, data: List[BatchRequestResponseDto]):
        for item in data:
            assert isinstance(item, BatchRequestResponseDto)
        return data

    def send(self, validated_data):
        result = self.process_item(validated_data)
        return result

    @property
    def update_state(self, *args, **kwargs) -> BatchRequestStatus:
        return BatchRequestStatus.done

    def check_run(self, is_ok, formatted_data):
        if not is_ok and self.is_batch_request:
            self.integration.batch_request.objects = None
            self.batch_service(self.integration.channel_id).to_fail(
                self.integration.batch_request)
        return False

    def get_channel_items_by_reference_object_ids(self, channel_response,
                                                  model_items_by_content,
                                                  integration_actions):
        product_ids = [str(item) for item in
                       model_items_by_content["productimage"]]

        model_items_by_content_product = self.get_products(product_ids)

        channel_items_by_product_id = {}
        for product_id, product in model_items_by_content_product.items():
            sku = self.get_barcode(obj=product)
            for channel_item in channel_response:
                # TODO: comment
                if channel_item.sku != sku:
                    continue
                remote_item = channel_item
                channel_items_by_product_id[product_id] = remote_item
                break
        return channel_items_by_product_id



from omnisdk.omnitron.client import OmnitronApiClient

from channel_app.core.integration import BaseIntegration
from channel_app.omnitron.batch_request import ClientBatchRequest
from channel_app.omnitron.commands.batch_requests import GetBatchRequests, \
    BatchRequestUpdate
from channel_app.omnitron.commands.error_reports import \
    CreateAddressErrorReports, \
    CreateErrorReports
from channel_app.omnitron.commands.integration_actions import \
    CreateIntegrationActions, \
    GetIntegrationActionsWithObjectId, GetIntegrationActionsWithRemoteId, \
    UpdateIntegrationActions, \
    GetIntegrationActions, GetObjectsFromIntegrationAction
from channel_app.omnitron.commands.orders.addresses import GetOrCreateAddress
from channel_app.omnitron.commands.orders.cargo_companies import GetCargoCompany
from channel_app.omnitron.commands.orders.customers import GetOrCreateCustomer
from channel_app.omnitron.commands.orders.orders import (
    CreateOrders,
    CreateOrderShippingInfo,
    GetOrders,
    ProcessOrderBatchRequests,
    CreateOrderCancel, GetCancellationRequest)
from channel_app.omnitron.commands.product_prices import (
    GetUpdatedProductPrices, ProcessPriceBatchRequests,
    GetInsertedProductPrices, GetInsertedProductPricesFromExtraPriceList,
    GetUpdatedProductPricesFromExtraPriceList)
from channel_app.omnitron.commands.product_stocks import (
    GetUpdatedProductStocks, ProcessStockBatchRequests,
    GetInsertedProductStocks, GetUpdatedProductStocksFromExtraStockList,
    GetInsertedProductStocksFromExtraStockList)
from channel_app.omnitron.commands.products import (
    GetInsertedProducts,
    GetUpdatedProducts,
    ProcessProductBatchRequests,
    GetDeletedProducts,
    ProcessDeletedProductBatchRequests,
    GetMappedProducts,
    GetProductPrices,
    GetProductStocks,
    GetInsertedOrUpdatedProducts,
    GetProductCategoryNodes, GetProductObjects, GetProductsFromBatchrequest,
    GetProductPricesWithOutCommit, GetProductStocksWithOutCommit,
    GetMappedProductsWithOutCommit)
from channel_app.omnitron.commands.setup import (
    CreateOrUpdateCategoryTreeAndNodes, CreateOrUpdateCategoryAttributes,
    GetCategoryIds, CreateOrUpdateChannelAttributeSet,
    GetOrCreateChannelAttributeSetConfig, CreateOrUpdateChannelAttribute,
    CreateOrUpdateChannelAttributeConfig, CreateOrUpdateChannelAttributeValue,
    GetOrCreateChannelAttributeValueConfig,
    AsyncCreateOrUpdateCategoryAttributes, GetOrCreateChannelAttributeSchema,
    UpdateChannelConfSchema, GetChannelAttributeSetConfigs,
    GetChannelAttributeSets)


class OmnitronIntegration(BaseIntegration):
    """
    Communicates with the Omnitron Api services through the commands defined. It manages
    OmnitronApiClient object on enter and exit methods.

    """
    actions = {
        "get_inserted_products": GetInsertedProducts,
        "get_updated_products": GetUpdatedProducts,
        "get_inserted_or_updated_products": GetInsertedOrUpdatedProducts,
        "get_deleted_products": GetDeletedProducts,
        "get_mapped_products": GetMappedProducts,
        "get_mapped_products_without_commit": GetMappedProductsWithOutCommit,
        "get_product_prices": GetProductPrices,
        "get_product_prices_without_commit": GetProductPricesWithOutCommit,
        "get_product_stocks": GetProductStocks,
        "get_product_stocks_without_commit": GetProductStocksWithOutCommit,
        "get_product_categories": GetProductCategoryNodes,
        "get_batch_requests": GetBatchRequests,
        "get_updated_stocks": GetUpdatedProductStocks,
        "get_inserted_stocks": GetInsertedProductStocks,
        "get_updated_stocks_from_extra_stock_list": GetUpdatedProductStocksFromExtraStockList,
        "get_inserted_stocks_from_extra_stock_list": GetInsertedProductStocksFromExtraStockList,
        "get_updated_prices": GetUpdatedProductPrices,
        "get_inserted_prices": GetInsertedProductPrices,
        "get_inserted_prices_from_extra_price_list": GetInsertedProductPricesFromExtraPriceList,
        "get_updated_prices_from_extra_price_list": GetUpdatedProductPricesFromExtraPriceList,
        "process_product_batch_requests": ProcessProductBatchRequests,
        "process_stock_batch_requests": ProcessStockBatchRequests,
        "process_price_batch_requests": ProcessPriceBatchRequests,
        "process_order_batch_requests": ProcessOrderBatchRequests,
        "process_delete_product_batch_requests": ProcessDeletedProductBatchRequests,
        "get_or_create_customer": GetOrCreateCustomer,
        "get_or_create_address": GetOrCreateAddress,
        "get_cargo_company": GetCargoCompany,
        "create_order": CreateOrders,
        "get_orders": GetOrders,
        "create_order_shipping_info": CreateOrderShippingInfo,
        "create_or_update_category_tree_and_nodes": CreateOrUpdateCategoryTreeAndNodes,
        "create_or_update_category_attributes": CreateOrUpdateCategoryAttributes,
        "create_or_update_category_attributes_async": AsyncCreateOrUpdateCategoryAttributes,
        "create_address_error_report": CreateAddressErrorReports,
        "create_error_report": CreateErrorReports,
        "get_integration_with_object_id": GetIntegrationActionsWithObjectId,
        "get_integration_with_remote_id": GetIntegrationActionsWithRemoteId,
        "get_integrations": GetIntegrationActions,
        "get_content_objects_from_integrations": GetObjectsFromIntegrationAction,
        "create_integration": CreateIntegrationActions,
        "update_integration": UpdateIntegrationActions,
        "get_category_ids": GetCategoryIds,
        "create_or_update_channel_attribute_set": CreateOrUpdateChannelAttributeSet,
        "get_or_create_channel_attribute_set_config": GetOrCreateChannelAttributeSetConfig,
        "create_or_update_channel_attribute": CreateOrUpdateChannelAttribute,
        "get_or_create_channel_attribute_schema": GetOrCreateChannelAttributeSchema,
        "create_or_update_channel_attribute_config": CreateOrUpdateChannelAttributeConfig,
        "get_channel_attribute_set_configs": GetChannelAttributeSetConfigs,
        "get_channel_attribute_set": GetChannelAttributeSets,
        "create_or_update_channel_attribute_value": CreateOrUpdateChannelAttributeValue,
        "get_or_create_channel_attribute_value_config": GetOrCreateChannelAttributeValueConfig,
        "batch_request_update": BatchRequestUpdate,
        "create_order_cancel": CreateOrderCancel,
        "update_channel_conf_schema": UpdateChannelConfSchema,
        "get_product_objects": GetProductObjects,
        "get_product_from_batch_request": GetProductsFromBatchrequest,
        "get_cancellation_requests": GetCancellationRequest,
        # "fetch_cancellation_plan": FetchCancellationPlan
    }

    def __init__(self, create_batch=True, content_type=None):
        """
        Some environment parameters are stored in the integration object for convenience.

        :param create_batch: Flag to decide whether a batch request to be created

        """
        from channel_app.core import settings
        self.create_batch = create_batch
        self.content_type = content_type
        if create_batch and not content_type:
            raise Exception("ContentType not defined")
        self.channel_id = settings.OMNITRON_CHANNEL_ID
        self.catalog_id = settings.OMNITRON_CATALOG_ID
        self.base_url = settings.OMNITRON_URL
        self.username = settings.OMNITRON_USER
        self.password = settings.OMNITRON_PASSWORD
        # TODO initialize api in init and check whether it is already initialized on enter method

    def __enter__(self):
        self.api = OmnitronApiClient(base_url=self.base_url,
                                     username=self.username,
                                     password=self.password)
        self.channel_is_active = self.channel.is_active
        if not self.channel_is_active:
            return
        if self.create_batch:
            self.batch_request = ClientBatchRequest(
                channel_id=self.channel_id).create()
            self.batch_request.content_type = self.content_type
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        del self.api
        if isinstance(exc_val, Exception) and not self.channel_is_active:
            return True



from setuptools import setup, find_packages

with open("README.md") as f:
    long_description = f.read()

setup(
    name="channel_app",
    version="0.0.101",
    packages=find_packages(),
    url="https://github.com/akinon/channel_app",
    description="Channel app for Sales Channels",
    long_description=long_description,
    long_description_content_type="text/markdown",
    author="akinonteam",
    python_requires=">=3.5",
    # We should pin the below to work with all the way from py27 to upto py39
    install_requires=["requests"],
    classifiers=[
        "Development Status :: 5 - Production/Stable",
    ],
)



from typing import List

from omnisdk.omnitron.endpoints import ChannelCargoEndpoint
from omnisdk.omnitron.models import CargoCompany

from channel_app.core.commands import OmnitronCommandInterface


class GetCargoCompany(OmnitronCommandInterface):
    endpoint = ChannelCargoEndpoint

    def get_cargo_company(self, data):
        cargo_company_code = self.objects
        for cargo_company in data:
            if cargo_company.erp_code == cargo_company_code:
                return cargo_company
        raise Exception("CargoCompany does not exists: {}".format(
            cargo_company_code))

    def get_data(self) -> List[CargoCompany]:
        """
        :return: [
            {
                "pk": 1,
                "name": "Yurtici Kargo",
                "erp_code": "yurtici-kargo",
                "shipping_company": "yurtici"
            },
        ]
        """
        params = getattr(self, "param_{}".format("params"), {})

        end_point = self.endpoint(channel_id=self.integration.channel_id)
        cargo_companies = end_point.list(
            params=params)
        for next_cargo_companies in end_point.iterator:
            cargo_companies.extend(next_cargo_companies)

        cargo_company = self.get_cargo_company(data=cargo_companies)
        return [cargo_company]



from channel_app.core.data import CategoryTreeDto, ErrorReportDto
from channel_app.core.settings import OmnitronIntegration, ChannelIntegration
from channel_app.omnitron.constants import ContentType


class SetupService(object):
    def create_or_update_category_tree_and_nodes(self, is_success_log=False):
        with OmnitronIntegration(
                content_type=ContentType.category_tree.value) as omnitron_integration:
            channel_integration = ChannelIntegration()
            category_tree, report, _ = channel_integration.do_action(
                key='get_category_tree_and_nodes',
                batch_request=omnitron_integration.batch_request)

            category_tree: CategoryTreeDto
            report: ErrorReportDto

            if report and (is_success_log or not report.is_ok):
                omnitron_integration.do_action(
                    key='create_error_report',
                    objects=report)

            omnitron_integration.do_action(
                key='create_or_update_category_tree_and_nodes',
                objects=category_tree)

    def create_or_update_category_attributes(self, is_success_log=False):
        with OmnitronIntegration(
                content_type=ContentType.attribute.value) as omnitron_integration:
            channel_integration = ChannelIntegration()
            category_integration_actions = omnitron_integration.do_action(
                key='get_category_ids')

            for category_ia in category_integration_actions:
                if not category_ia.remote_id:
                    continue

                category, report, data = channel_integration.do_action(
                    key='get_category_attributes',
                    objects=category_ia,
                    batch_request=omnitron_integration.batch_request
                )
                if report and (is_success_log or not report.is_ok):
                    omnitron_integration.do_action(
                        key='create_error_report',
                        objects=report)

                omnitron_integration.do_action(
                    key='create_or_update_category_attributes',
                    objects=category)

    def update_channel_conf_schema(self):
        with OmnitronIntegration(
                content_type=ContentType.channel.value) as omnitron_integration:
            channel_integration = ChannelIntegration()
            schema, _, _ = channel_integration.do_action(
                key='get_channel_conf_schema',
                batch_request=omnitron_integration.batch_request)

            omnitron_integration.do_action(key="update_channel_conf_schema",
                                           objects=schema)



import datetime
from dataclasses import dataclass
from decimal import Decimal
from typing import List, Optional

from channel_app.omnitron.constants import ResponseStatus, \
    ChannelConfSchemaDataTypes


@dataclass
class CategoryNodeDto:
    name: str
    children: List['CategoryNodeDto']
    remote_id: Optional[str] = ''
    parent: Optional['CategoryNodeDto'] = None


@dataclass
class CategoryTreeDto:
    root: CategoryNodeDto


@dataclass
class CategoryAttributeValueDto:
    remote_id: str
    name: str


@dataclass
class CategoryAttributeDto:
    remote_id: str
    name: str
    required: bool
    variant: bool
    allow_custom_value: bool
    values: List[CategoryAttributeValueDto]


@dataclass
class CategoryDto:
    remote_id: str
    name: str
    attributes: List[CategoryAttributeDto]


@dataclass
class ChannelConfSchemaField:
    required: bool
    data_type: ChannelConfSchemaDataTypes
    key: str
    label: str
    schema: Optional[dict] = None


@dataclass
class ErrorReportDto:
    action_content_type: str
    action_object_id: int
    modified_date: str
    raw_request: str = ""
    raw_response: str = ""
    error_code: str = "custom"
    error_description: str = "custom"
    is_ok: bool = False
    target_content_type: Optional[str] = ''
    target_object_id: Optional[str] = ''


@dataclass
class BatchRequestObjectsDto:
    pk: int
    version_date: str
    content_type: str
    failed_reason_type: Optional[str] = ''
    remote_id: Optional[int] = 0


class BatchRequestDto:
    channel: int
    local_batch_id: str
    content_type: str
    pk: Optional[int] = 0
    remote_batch_id: Optional[str] = ''
    status: Optional[str] = ''
    objects: Optional[List[BatchRequestObjectsDto]] = []


@dataclass
class ProductBatchRequestResponseDto:
    status: ResponseStatus
    sku: str
    remote_id: Optional[str] = ''
    message: Optional[str] = ''


@dataclass
class BatchRequestResponseDto:
    status: ResponseStatus
    remote_id: Optional[str] = ''
    sku: Optional[str] = ''
    message: Optional[str] = ''


@dataclass
class OrderBatchRequestResponseDto:
    status: ResponseStatus
    remote_id: str
    number: str
    message: Optional[str] = ''


@dataclass
class OmnitronOrderDto:
    remote_id: str
    number: str
    channel: int
    customer: int
    shipping_address: int
    billing_address: int
    currency: str
    amount: Decimal
    shipping_amount: Decimal
    shipping_tax_rate: Decimal
    extra_field: dict
    cargo_company: int
    created_at: datetime.datetime
    delivery_type: Optional[str] = None
    discount_amount: Optional[Decimal] = "0.0"
    net_shipping_amount: Optional[Decimal] = "0.0"
    tracking_number: Optional[str] = None
    carrier_shipping_code: Optional[str] = ""
    remote_addr: Optional[str] = None
    has_gift_box: Optional[bool] = False
    gift_box_note: Optional[str] = ""
    client_type: Optional[str] = ""
    language_code: Optional[str] = ""
    notes: Optional[str] = ""
    delivery_range: Optional[str] = None
    shipping_option_slug: Optional[str] = ""
    status: Optional[str] = ""


@dataclass
class OrderItemDto:
    remote_id: str
    product: str
    price_currency: str
    price: Decimal
    tax_rate: Decimal
    extra_field: dict
    status: Optional[str] = None
    price_list: Optional[int] = None
    stock_list: Optional[int] = None
    tracking_number: Optional[str] = None
    carrier_shipping_code: Optional[str] = ""
    discount_amount: Optional[Decimal] = 0.0
    retail_price: Optional[Decimal] = 0.0
    attributes: Optional[dict] = None
    attributes_kwargs: Optional[dict] = None
    parent: Optional[str] = None
    delivered_date: Optional[str] = None
    estimated_delivery_date: Optional[str] = None


# dataclass used for omnitron.commands.orders.orders.CreateOrders
@dataclass
class OmnitronCreateOrderDto:
    order: OmnitronOrderDto
    order_item: List[OrderItemDto]


@dataclass
class CancelOrderDto:
    order: str  # remote order number
    cancel_items: List[str]  # order_item_remote_id list
    reasons: dict  # order_item_remote_id : reason code
    is_cargo_refund: Optional[bool] = False  # default False
    forced_refund_amount: Optional[bool] = None  # default False
    refund_invoice_number: Optional[str] = None


@dataclass
class CustomerDto:
    email: str  # "john.doe@akinon.com"
    first_name: str  # "John"
    last_name: str  # "Doe"
    channel_code: str  # will be unique
    extra_field: Optional[dict] = None
    phone_number: Optional[str] = None  # "05556667788"
    is_active: Optional[bool] = True


@dataclass
class AddressDto:
    email: str  # "john.doe@akinon.com"
    phone_number: str  # "05556667788"
    first_name: str  # "John"
    last_name: str  # "Doe"
    country: str  # 1
    city: str  # 80
    line: str  # "Hemen sahil kenarı"
    title: Optional[str] = None  # "COMM-876"
    township: Optional[str] = None  # 933
    district: Optional[str] = None  # 71387
    postcode: Optional[str] = None  # ""
    notes: Optional[str] = None  # null
    company_name: Optional[str] = None  # ""
    tax_office: Optional[str] = None  # ""
    tax_no: Optional[str] = None  # ""
    e_bill_taxpayer: Optional[bool] = False
    remote_id: Optional[str] = None  # null
    identity_number: Optional[str] = None  # null
    extra_field: Optional[dict] = None  # {}
    is_active: Optional[bool] = True  # true
    retail_store: Optional[str] = None


@dataclass
class ChannelOrderDto(OmnitronOrderDto):
    customer = CustomerDto
    shipping_address = AddressDto
    billing_address = AddressDto
    cargo_company = str


@dataclass
class ChannelCreateOrderDto:
    order: ChannelOrderDto
    order_item: List[OrderItemDto]



from typing import List

from omnisdk.omnitron.endpoints import ChannelIntegrationActionEndpoint, \
    ChannelProductPriceEndpoint, ChannelBatchRequestEndpoint, \
    ChannelExtraProductPriceEndpoint, ChannelExtraProductStockEndpoint
from omnisdk.omnitron.models import ProductPrice

from channel_app.core.commands import OmnitronCommandInterface
from channel_app.core.data import BatchRequestResponseDto
from channel_app.core.utilities import split_list
from channel_app.omnitron.commands.batch_requests import ProcessBatchRequests
from channel_app.omnitron.constants import (ContentType, BatchRequestStatus,
                                            IntegrationActionStatus,
                                            FailedReasonType)


class GetUpdatedProductPrices(OmnitronCommandInterface):
    endpoint = ChannelProductPriceEndpoint
    path = "updates"
    BATCH_SIZE = 100
    content_type = ContentType.product_price.value

    def get_data(self) -> List[ProductPrice]:
        prices = self.get_product_prices()
        prices = self.get_integration_actions(prices)
        return prices

    def get_product_prices(self) -> List[ProductPrice]:
        prices = self.endpoint(
            path=self.path,
            channel_id=self.integration.channel_id
        ).list(
            params={
                "limit": self.BATCH_SIZE
            }
        )
        prices = prices[:self.BATCH_SIZE]
        objects_data = self.create_batch_objects(data=prices,
                                                 content_type=self.content_type)
        self.update_batch_request(objects_data=objects_data)
        return prices

    def get_integration_actions(self, prices: List[ProductPrice]):
        if not prices:
            return []
        endpoint = ChannelIntegrationActionEndpoint(
            channel_id=self.integration.channel_id)
        price_integration_actions = endpoint.list(
            params={
                "local_batch_id": self.integration.batch_request.local_batch_id,
                "status": IntegrationActionStatus.processing,
                "channel_id": self.integration.channel_id
            })
        for price_batch in endpoint.iterator:
            price_integration_actions.extend(price_batch)
        price_ia_dict = {ia.object_id: ia for ia in price_integration_actions}
        for price in prices:
            price_ia = price_ia_dict[price.pk]
            price.remote_id = price_ia.remote_id
        return prices


class GetInsertedProductPrices(GetUpdatedProductPrices):
    path = "inserts"

    def get_integration_actions(self, prices: List[ProductPrice]):
        if not prices:
            return []
        endpoint = ChannelIntegrationActionEndpoint(
            channel_id=self.integration.channel_id)
        product_ids = [str(price.product) for price in prices]
        product_ias = endpoint.list(
            params={"object_id__in": ",".join(product_ids),
                    "content_type_name": ContentType.product.value,
                    "status": IntegrationActionStatus.success,
                    "channel_id": self.integration.channel_id
                    })
        for product_batch in endpoint.iterator:
            product_ias.extend(product_batch)
        product_integrations_by_id = {ia.object_id: ia for ia in product_ias}

        for price in prices:
            if price.product in product_integrations_by_id:
                product_ia = product_integrations_by_id[price.product]
                price.remote_id = product_ia.remote_id
            else:
                price.failed_reason_type = FailedReasonType.channel_app.value
                self.failed_object_list.append(
                    (price, ContentType.product_price.value,
                     "Product has not been sent"))
        return prices


class GetInsertedProductPricesFromExtraPriceList(OmnitronCommandInterface):
    endpoint = ChannelExtraProductPriceEndpoint
    path = "inserts"
    BATCH_SIZE = 100
    content_type = ContentType.product_price.value

    def get_data(self) -> List[ProductPrice]:
        self.price_list_id = self.objects
        prices = self.get_product_prices()
        prices = self.get_integration_actions(prices)
        return prices

    def get_product_prices(self) -> List[ProductPrice]:
        endpoint = self.endpoint(path=self.path,
                                 channel_id=self.integration.channel_id)
        prices = endpoint.list(
            params={"price_list": self.price_list_id,
                    "limit": self.BATCH_SIZE}
        )

        prices = prices[:self.BATCH_SIZE]
        objects_data = self.create_batch_objects(data=prices,
                                                 content_type=self.content_type)
        self.update_batch_request(objects_data=objects_data)
        return prices

    def get_integration_actions(self, prices: List[ProductPrice]):
        if not prices:
            return []
        endpoint = ChannelIntegrationActionEndpoint(
            channel_id=self.integration.channel_id)
        price_integration_actions = endpoint.list(
            params={
                "local_batch_id": self.integration.batch_request.local_batch_id,
                "status": IntegrationActionStatus.processing
            })
        for price_batch in endpoint.iterator:
            price_integration_actions.extend(price_batch)
        price_ia_dict = {ia.object_id: ia for ia in price_integration_actions}
        for price in prices:
            price_ia = price_ia_dict[price.pk]
            price.remote_id = price_ia.remote_id
        return prices


class GetUpdatedProductPricesFromExtraPriceList(
    GetInsertedProductPricesFromExtraPriceList):
    path = "updates"


class GetProductStocksFromProductPrices(OmnitronCommandInterface):
    endpoint = ChannelExtraProductStockEndpoint
    content_type = ContentType.product_stock.value
    CHUNK_SIZE = 50

    def get_data(self) -> List[ProductPrice]:
        product_prices = self.objects
        self.get_product_stock(product_prices)
        return product_prices

    def normalize_response(self, data, response) -> List[object]:
        object_list = []
        failed_prices = [failed_product_prices[0] for failed_product_prices in
                         self.failed_object_list]
        product_price_object_list = self.create_batch_objects(
            data=failed_prices,
            content_type=ContentType.product_price.value)
        object_list.extend(product_price_object_list)

        self.create_integration_actions(data, object_list)
        self.update_batch_request(object_list)
        return data

    def create_integration_actions(self, data, object_list):
        commit_product_stocks = [product_prices.productstock for product_prices
                                 in data
                                 if not getattr(product_prices,
                                                "failed_reason_type",
                                                None)]
        product_stock_object_list = self.create_batch_objects(
            data=commit_product_stocks,
            content_type=ContentType.product_stock.value)
        object_list.extend(product_stock_object_list)

    def get_product_stock(self, product_prices: List[ProductPrice]) -> List[
        ProductPrice]:
        if not product_prices:
            empty_list: List[ProductPrice] = []
            return empty_list

        product_ids = [str(pp.product) for pp in product_prices if
                       not getattr(pp, "failed_reason_type", None)]

        endpoint = self.endpoint(channel_id=self.integration.channel_id)
        stocks = []
        for chunk in split_list(product_ids, self.CHUNK_SIZE):
            stock_batch = self.get_stocks(chunk, endpoint)
            if not stock_batch:
                break
            stocks.extend(stock_batch)

        product_stocks = {s.product: s for s in stocks}

        for index, product_price in enumerate(product_prices):
            if getattr(product_price, "failed_reason_type", None):
                continue
            try:
                product_price.productstock = product_stocks[
                    product_price.product]
            except KeyError:
                product_price.failed_reason_type = FailedReasonType.channel_app.value
                self.failed_object_list.append(
                    (product_price, ContentType.product_price.value,
                     "StockNotFound"))
                continue

        return product_prices

    def get_stocks(self, chunk, endpoint):
        stock_list = getattr(self, "param_stock_list",
                             self.integration.catalog.stock_list)
        stock_batch = endpoint.list(params={"product__pk__in": ",".join(chunk),
                                            "stock_list": stock_list})
        return stock_batch


class ProcessPriceBatchRequests(OmnitronCommandInterface, ProcessBatchRequests):
    endpoint = ChannelBatchRequestEndpoint
    content_type = ContentType.product_price.value
    CHUNK_SIZE = 50
    BATCH_SIZE = 100

    def get_data(self):
        return self.objects

    def validated_data(self, data: List[BatchRequestResponseDto]):
        for item in data:
            assert isinstance(item, BatchRequestResponseDto)
        return data

    def send(self, validated_data):
        result = self.process_item(validated_data)
        return result

    @property
    def update_state(self, *args, **kwargs) -> BatchRequestStatus:
        return BatchRequestStatus.done

    def check_run(self, is_ok, formatted_data):
        if not is_ok and self.is_batch_request:
            self.integration.batch_request.objects = None
            self.batch_service(self.integration.channel_id).to_fail(
                self.integration.batch_request)
        return False

    def get_channel_items_by_reference_object_ids(self, channel_response,
                                                  model_items_by_content,
                                                  integration_actions):
        product_ids = [str(item) for item in
                       model_items_by_content["productprice"]]

        model_items_by_content_product = self.get_products(product_ids)

        channel_items_by_product_id = {}
        for product_id, product in model_items_by_content_product.items():
            sku = self.get_barcode(obj=product)
            for channel_item in channel_response:
                # TODO: comment
                if channel_item.sku != sku:
                    continue
                remote_item = channel_item
                channel_items_by_product_id[product_id] = remote_item
                break
        return channel_items_by_product_id
